#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "utility.h"

#define USAGE "USAGE: rescale_tree [-v] scale treefile\n"

#define MAX_CHILDREN 16
#define MAX_WORD 64

typedef struct NodeS {
  struct NodeS *children[MAX_CHILDREN];
  int n_children;
  double time_to_parent;
  double time_variance;
  char name[MAX_WORD];
} Node;

typedef struct {
  Node* root;
  double scale;
  double time;
  int variance;
} Params;

void print_tree(Params* params);
void print_node(Params* params, Node* node, int is_root);
void read_tree(Params* params, char* treefile);
Node* read_node(Params* params, FILE* in, int is_root);


int main(int argc, char* argv[]) {
  Params params;
  int i;

  if(argc == 4 && !strcmp(argv[1], "-v")) {
    params.variance = 1;
  }
  else {
    argc == 3 || die(USAGE);
    params.variance = 0;
  }
  params.scale = atof(argv[params.variance + 1]);
  read_tree(&params, argv[params.variance + 2]);
  params.time = 0.0;
  print_tree(&params);
  fprintf(stderr, "total time = %g\n", params.time);
}

void read_tree(Params* params, char* treefile) {
  FILE* in;

  (in = fopen(treefile, "r")) || die("Could not open treefile: %s\n", treefile);
  params->root = read_node(params, in, 1);
}

Node* read_node(Params* params, FILE* in, int is_root) {
  Node* node;
  int i, ch;

  (node = (Node*)malloc(sizeof(Node))) || die("Could not allocate memory for node\n");
  node->n_children = 0;

  while((ch = getc(in)) == '\n');
  if(ch == '(') {
    while(1) {
      while(isspace(ch = getc(in)));
      if(ch == ')')
	break;
      node->n_children < MAX_CHILDREN || die("Node has too many children\n");
      if(node->n_children)
	ch == ',' || die("Missing comma in treefile\n");
      else
	ungetc(ch, in);
      node->children[node->n_children++] = read_node(params, in, 0);
    }
  }
  else {
    ungetc(ch, in);
    fscanf(in, "%[^\n:].63", &node->name) == 1 || die("Missing name in treefile\n");
  }
  if(!is_root) {
    while(isspace(ch = getc(in)));
    ch == ':' || die("Missing colon in treefile after %s\n", node->name);
    while(isspace(ch = getc(in)));
    ungetc(ch, in);
    fscanf(in, "%lg", &node->time_to_parent) == 1 || die("Missing time to parent\n"); 
    if(params->variance)
      fscanf(in, "+/-%lg", &node->time_variance) == 1 || die("Missing variance\n");
  }
  else {
    while(isspace(ch = getc(in)));
    ch == ';' || die("Missing semicolon at end of treefile: %c\n", ch);
  }
  return node;
}

void print_tree(Params* params) {
  if(params->root)
    print_node(params, params->root, 1);
}

void print_node(Params* params, Node* node, int is_root) {
  int n;
  double time;

  if(node->n_children) {
    printf("(");
    for(n = 0; n < node->n_children; n++) {
      if(n)
	printf(",");
      print_node(params, node->children[n], 0);
    }
    printf(")");
  }
  else
    printf(node->name);
  if(!is_root) {
    printf(":%g", time = node->time_to_parent * params->scale);
    if(params->variance)
      printf("+/-%g", node->time_variance * params->scale); 
    params->time += time;
  }
  else
    printf(";\n");
}



